
#' Given a fitted hierarchy of models (like returned by \code{fitNested}
#' assign the alignment to a unique model.
#'
#' @param fitobj The fitted hierachy of models (see \code{\link{fitNested}}).
#' @param mvecMat Matrix (logical) of model-vectors (used to derive fitobj).
#'        Rows correspond to model parameters, columns enumerate the models, entries
#'        correspond to whether a parameter is active in a model. Column names must
#'        match the model names in \code{fitobj}.
#' @param pcut P-value cutoff to consider significant
#' @param crit character Criterion for model selection in case contenders are not nested.
#'        Possible values are "aic" (default), "aicc" (second order aic) and "bic".
#' @return Either \code{NA}, or a list with the following components:
#' * string  The resulting fitted model (in string representation, zero or one for each parameter (row of mvecMat))
#' * pval The p-value of this model with respect to the best-fitting, most complex nexted model
#' * like The likelihood of the fitted model
#' * likeNull The likelihood of the null model
#' * crit The criterion used to assign a model class to the alignment. One of Either
#' * modNested The best-fitting model nested inside the assigned model
#' * stringe Extended string representation: each parameter name followed by TRUE/FALSE;
#' \code{'unique'}, \code{'aic'} or \code{'bic'}.
#'
#' @importClassesFrom graph graphAM
#' @importFrom graph degree nodes removeNode
#' @importFrom methods new
#' @export
classifyNested <- function(fitobj,mvecMat,pcut=0.01,crit="aic"){
#====================================================================

  #- bookkeeping
  if(!crit %in% c("aic","aicc","bic")) stop("invalid model selection criterion.")

  #- check that the fitobj and mvecMat are aligned
  if( sum( ! names(fitobj) %in% colnames(mvecMat) ) > 0 ) stop("fitobj and mvecMat not consistent")
  mvecMat = mvecMat[,names(fitobj)]

  #- check the matrix of model vectors and make rownames if needed
  if(!( is.matrix(mvecMat) & is.logical(mvecMat)  )){
    stop("mvecMat needs to be logical matrix")
  }
  if(is.null(rownames(mvecMat))) rownames(mvecMat) = paste("par_",1:nrow(mvecMat),collapse="")

	#- null model did not fit
	if(is.na(fitobj[[1]]$likelihood)){
              return(NA)
	}

	#- remove nodes with non-significant p-values
	pvs = unlist(lapply(fitobj,function(x) {if(any(is.na(x))) return(NA) ; return(x$pval)}))
	drop.ind = c(which(  is.na(pvs) | pvs>pcut  ))
	fo = fitobj[-drop.ind]

	pvs[is.na(pvs)]=1
	if(sum(pvs<1) == 0){
		minplik = fitobj[[1]]$likelihood
	} else {
		minplik = fitobj[[which.min(pvs)]]$likelihood
	}
	if(sum(pvs<=pcut) ==0){
		string    = names(fitobj)[1]
                pval      = min(pvs)
                like      = minplik
                like.null = fitobj[[1]]$likelihood
                crit      = "unique"
                nmod      = fitobj[[1]]$nestedMod
                mind      = which(colnames(mvecMat)==string)
                stringe   = paste(names(mvecMat[,mind]),mvecMat[,mind],sep=":",collapse="|")
                #return(c(string,pval,like,like.null,crit,nmod))
                return(list(string=string,pval=pval,like=like,likeNull = like.null,crit=crit,mod=nmod,stringe=stringe))
	}
	#- here we (1) have a fitted model and (2) at least one decent p-value
	minpval = min(pvs)
	minplik = fitobj[[which.min(pvs)]]$likelihood
	kept.inds  = which(pvs <=pcut)
	kept.names = names(fitobj)[kept.inds]

	#- make the adjacency matrix and graph
	tmp  = expand.grid(1:length(kept.inds),1:length(kept.inds))
	tmp2 = apply(tmp,1,function(x) modNest(mvecMat[,kept.inds[x[1]]],mvecMat[,kept.inds[x[2]]]))
	amat = matrix(as.integer(tmp2),nrow=length(kept.inds))
	diag(amat) = 0
	colnames(amat)=kept.names
	rownames(amat)=kept.names
	gg   =  new("graphAM", amat,edge="directed") #- from the graph package


	#- remove nodes nested in more complex models
	drop.nodes = names(degree(gg)$outDegree)[degree(gg)$outDegree >0]
	if(length(drop.nodes) == length(nodes(gg))-1){
		cand = nodes(gg)[!(nodes(gg)%in% drop.nodes)]
	} else {
		ggg = gg
		if(length(drop.nodes>0)) ggg = removeNode( drop.nodes,ggg)
		cand = nodes(ggg) #- now the names of amat, therefore names of fitobj
	}
	#= choose model
	if(length(cand) == 0){
		string    = names(fitobj)[1]
    pval      = minpval
    like      = minplik
    like.null = fitobj[[1]]$likelihood
    crit      = "unique"
    nmod      = fitobj[[1]]$nestedMod
    mind      = which(colnames(mvecMat)==string)
    stringe   = paste(names(mvecMat[,mind]),mvecMat[,mind],sep=":",collapse="|")
    return(list(string=string,pval=pval,like=like,likeNull = like.null,crit=crit,modNested=nmod,stringe=stringe))
	}
	if(length(cand) == 1){
		string    = cand
		pval      = fo[[cand]]$pval
		like      = fo[[cand]]$likelihood
		like.null = fitobj[[1]]$likelihood
		crit      = "unique"
    nmod      = fo[[cand]]$nestedMod
    mind      = which(colnames(mvecMat)==string)
    stringe   = paste(names(mvecMat[,mind]),mvecMat[,mind],sep=":",collapse="|")
		#return(c(string,pval,like,like.null,crit,nmod))
    return(list(string=string,pval=pval,like=like,likeNull = like.null,crit=crit,modNested=nmod,stringe=stringe))
	} else {
		if(crit == "aic")  crits = lapply(fo[cand],function(x)x$aic)
    if(crit == "aicc") crits = lapply(fo[cand],function(x)x$aicc)
    if(crit == "bic")  crits = lapply(fo[cand],function(x)x$bic)
    cand      = names(crits)[which.min(crits)]
		string    = cand
    pval      = fo[[cand]]$pval
    like      = fo[[cand]]$likelihood
    like.null = fitobj[[1]]$likelihood
    nmod      = fo[[cand]]$nestedMod
    mind      = which(colnames(mvecMat)==string)
    stringe   = paste(names(mvecMat[,mind]),mvecMat[,mind],sep=":",collapse="|")
    return(list(string=string,pval=pval,like=like,likeNull = like.null,crit=crit,modNested=nmod,stringe=stringe))
	}
}
