
#' Fit a nested hierarchy of models to an MSA
#'
#' @param msa Multiple sequence alignment
#' @param mvecMat Matrix (logical) of model-vectors, fist column must be all  FALSE.
#'        Rows correspond to model parameters, columns enumerate the models, entries
#'        correspond to whether a parameter is active in a model.
#' @param NULL.MOD The (null) tree-model (no sel, bgc on any branch)
#' @param amat Adajency matrix containing nested-by-one-parameter relatinshiops between models
#' @param PCUT P-value cutoff to use for stop traversing
#'
#' @return A list with \code{ncol(mvecMat)} elements. Each element in this list corresponds
#' to the respective model in mvecMat and is phylogenetic tree model as returned from the
#' \code{phyloFit} function in the \code{rphast} packaage. For models that were not fit, the
#' corresponding list element is \code{NA}. For all fitted models except the null model (the
#' first element in the list) the model as been augmented with the following components:
#' * pval The P-value with respect to the best-fitting, most-complex nested model. NA if not available.
#' * aic The Akaike information criterion. NA if not available.
#' * aicc Second order AIC, meant to account for small sample size. NA if not available.
#' * bic  The Bayesian infromation criterion. NA if not available.
#' * nestedMod The best-fitting nested model within the current model. NA if not available.
#'
#' @import rphast
#' @importFrom stats pchisq
#' @export
fitNested <- function(msa, mvecMat, NULL.MOD, amat=NULL, PCUT=0.01){
#=================================================================

	if(is.null(amat)){
		amat = getAdjacency(mvecMat)
	} else {
		if( !all(colnames(amat) == colnames(mvecMat)) ){
		   stop("Error: mvecMat and amat do not match.")
		}
	}

	FITTED        = as.list(rep(NA,ncol(mvecMat)))
	names(FITTED) = colnames(mvecMat)
	nsites        = ninf.msa(msa)
	npars         = colSums(mvecMat)+1

	pvalfu <- function(x, df=1) { res = pchisq(x,df=df,lower.tail=FALSE)/2 ; if(x==0) res = res+1/2 ; return(res)}

	#- loop through models
	for(i in 1:dim(amat)[2]){
		if(i==1){
			MOD         = mvec2mod(mvecMat[,i],NULL.MOD)
			FITTED[[i]] = fit.mod(MOD,msa) #- global selection parameter in the null model
			if(any(is.na(FITTED[[i]]))) return(FITTED) #-  null model did not fit, EXIT here
      FITTED[[i]]$aic = NA
      FITTED[[i]]$aicc = NA
      FITTED[[i]]$bic = NA
      FITTED[[i]]$pval = NA
      FITTED[[i]]$nestedMod = NA
      FITTED[[i]]$likelihood = round(FITTED[[i]]$likelihood,2)
			MIN.SEL = 0 #- on ls-model selection relative to main model. We only look for acc
			MIN.LKI = round(FITTED[[i]]$likelihood,2)
		}else{
			parent.inds = which(amat[-1,i] >0) #- exclude null model
			if(length(parent.inds) == 0){
				MOD         = mvec2mod(mvecMat[,i],NULL.MOD,minSel=MIN.SEL)
				FITTED[[i]]      = fit.mod(MOD,msa)
				if(!any(is.na(FITTED[[i]]))){ #- fitting worked
					FITTED[[i]]$likelihood   = round(FITTED[[i]]$likelihood,2)
					FITTED[[i]]$pval         = pvalfu(max(0,2*(FITTED[[i]]$likelihood - FITTED[[1]]$likelihood)))
					FITTED[[i]]$aic          = -2/nsites*FITTED[[i]]$likelihood + 2*npars[i]
          FITTED[[i]]$aicc      = FITTED[[i]]$aic + 2*(npars[i]*(npars[i]+1)) / (nsites - npars[i] -1)
					FITTED[[i]]$bic          = -2*FITTED[[i]]$likelihood + log(nsites)*npars[[i]]
          FITTED[[i]]$nestedMod    = colnames(amat)[1]
				}
			}else{
				parent.inds = parent.inds + 1 #- because null model was gone
				tmp = FITTED[parent.inds]
				#- drop this model if all nested models are NAs
				if(all(is.na(tmp))) next
				tmp = tmp[!is.na(tmp)]
				pvs = unlist(lapply(tmp,function(x) x$pval))
				if(min(pvs)<=PCUT){
					MOD         = mvec2mod(mvecMat[,i],NULL.MOD,minSel=MIN.SEL)
					FITTED[[i]] = fit.mod(MOD,msa)
					if(!any(is.na(FITTED[[i]]))){ #- fitting worked
						FITTED[[i]]$likelihood = round(FITTED[[i]]$likelihood,2)
						likes       = unlist(lapply(tmp,function(x) x$likelihood))
						m.like      = max(likes)
						FITTED[[i]]$pval = pvalfu(max(0,2*(FITTED[[i]]$likelihood - m.like)))
						if(FITTED[[i]]$likelihood < m.like) FITTED[[i]]$pval = 1
						FITTED[[i]]$aic       = -2/nsites*FITTED[[i]]$likelihood + 2*npars[i]
            FITTED[[i]]$aicc      = FITTED[[i]]$aic + 2*(npars[i]*(npars[i]+1)) / (nsites - npars[i] -1)
						FITTED[[i]]$bic       = -2*FITTED[[i]]$likelihood + log(nsites)*npars[[i]]
            FITTED[[i]]$nestedMod = colnames(amat)[parent.inds][!is.na(tmp)][which.max(likes)]
					}
				} else {
					FITTED[[i]] = NA
				}
			}
		}
	}
	return(FITTED)
}
