
#' Convert integer to a string containing binary representationsm
#'
#' @param x The interger to convert
#' @param bits The number of bits to consider in the string
#'
#' @export
int2bitStr <- Vectorize( function(x,bits){
#=========================================

	paste(rev(as.integer(intToBits(as.integer(as.integer(x))))[1:bits]), collapse="")
} )

#' Convert integer to a binary vector, each entry representing a bit.
#'
#' @param x The interger to convert
#' @param bits The number of bits to consider in the string
#'
#' @export
int2bitVec <- Vectorize (function(x,bits){
#=========================================

	rev(as.integer(intToBits(as.integer(as.integer(x))))[1:bits])
} )


######################################################
######################################################
#
#  Models are partially respesented by logical vectors, where
#  each entry indicates the presence/absence of a
#  parameter.
#  Specifically, we assyme a number of species, N
#  and for each species two parameters, selection and
#  GC-biased gene conversion, so the encoding is like
#
#	spec1.sel	T/F
#	spec1.bgc	T/F
#	spec2.sel	T/F
#	spec2.bgc	T/F
#	spec3.sel	T/F
#	...
#
#  The names attribute of the vector contains the
#  species and the parameter type to make it unique
#  (it is assumed to bealways sel first bgc second).
######################################################
#######################################################





#' Check nestedness of models in vector representation
#'
#' @param vec.i Vector representation of model i
#' @param vec.j Vector representation of model j
#'
#' @export
modNest <- function(vec.i, vec.j){
#=================================

	#- vec.i has no ones wehre vec.j does not:
	sum(vec.i | vec.j) == sum(vec.j)

}

#' Check nestedness of models in vector representation
#'
#' @param vec.i Vector representation of model i
#' @param vec.j Vector representation of model j
#'
#' @export
modNestByOne <- function(vec.i, vec.j){
#======================================

	 #- vec.j has one one where vec.i does have a zero:
	 (sum(vec.i & vec.j) == sum(vec.i)) & (sum(vec.j) == sum(vec.i)+1)
}


#' Take a model-vector and expand it into a parameter list
#'
#' This is meant to work with a phastCons model, placing
#' (combinations of) selection and GC-biased gene conversion
#' in branches as specified by the model-vector
#'
#' @param mvec Model-vector
#' @param minSel The minimum selection (default zero, looking for acceleration)
#' @param maxSel The maximum selection (empty, was needed to make rphast work)
#'
mvec2parList <- function(mvec,minSel=0,maxSel=""){
#===================================================

	#- the species with parameters
	specVec = unlist(lapply(strsplit(names(mvec),split="\\."),"[[",1))
	spec = unique(specVec[as.logical(mvec)])

	sep = vector(length(spec),mode="list")
	names(sep) = spec
	for(s in spec){
		spec.bits = mvec[specVec==s]
		if(all(spec.bits == c(1,0))) 	sep[[s]] =   paste("sel[",minSel,",",maxSel,"]",sep="")
		if(all(spec.bits == c(0,1)))	sep[[s]] = "bgc[0,]"
		if(all(spec.bits == c(1,1)))	sep[[s]] = c(paste("sel[",minSel,",",maxSel,"]",sep=""),"bgc[0,]")
	}

	return(list(spec=spec, sep=sep))
}


#' Expand a phast tree-model to contain sel/bgc on brances
#'
#' Add the parameter list to the given model
#'
#' @param null.mod the tree model
#' @param parList parameter list (see mvec2parList)
#'
#' @import rphast
expandMod <- function(null.mod, parList){
#=========================================

	specs = parList$spec
	seps  = parList$sep

	mod = null.mod
	mod$selection=0
	for(spec in specs){
		mod = add.ls.mod(mod,spec,separate.params=seps[[spec]],const.params=c("ratematrix","backgd"))
	}
	return(mod)
}

#' Derive an expanded model from a null model and an mvec
#'
#' @param mvec Model-vector
#' @param null.mod Tree model
#' @param ... Additional arguments to mvec2parList
#'
#' @import rphast
mvec2mod <- function(mvec, null.mod, ...){
#========================================

	parList = mvec2parList(mvec,...)
	mod     = expandMod(null.mod,parList)
	return(mod)
}

#' For a matrix of Model-vectors, indicate nested-by-one-parameter relationships
#'
#' @param mvecMat Matrix, each column a model-vector
#'
#' @export
getAdjacency <- function(mvecMat){
#=================================
	nmods = ncol(mvecMat)
	tmp  = expand.grid(1:nmods,1:nmods)
	tmp2 = apply(tmp,1,function(x) modNestByOne(mvecMat[,x[1]],mvecMat[,x[2]]))
	amat = matrix(as.integer(tmp2),nrow=nmods)
	diag(amat) = 0
	colnames(amat)=colnames(mvecMat)
	rownames(amat)=colnames(amat)
	return(amat)
}

#' Fit a model using phyloFit
#'
#' Fits only selection and bgc-parameters
#'
#' @param mod The model to fit
#' @param msa The sequence alignment
#' @param quiet phyloFit quiet parameter
#' @param ... Additional phyloFit parameters
#'
#' @import rphast
fit.mod <- function(mod,msa,quiet=TRUE,...){
#===========================================

	res = try(	rphast::phyloFit(	msa,
				init.mod = mod,
				no.opt=c("ratematrix","backgd","branches"),
				ninf.sites=20,
				quiet=quiet,...),
			silent=TRUE)
	if( inherits(res,"try-error") ) return(NA) else return(res) ;
}
